#!/perl -w
# NAME: cvsversion.pl
# AIM: To examine an input directory, seeking CVS folders and files,
# and reporting when the last update was done.
# 15/08/2007 - geoff mclane - geoffair.net/mperl
use strict;
use warnings;
use File::stat; # to get the file date
use File::Basename;
use Time::gmtime;
### prt( "$0 ... Hello, World ...\n" );
my $in_folder = 'C:/FGCVS/xmlrpc-c';
###my $in_folder = 'C:/FGCVS/Tidy';
my @fnd_files = ();
my @fnd_cvs = ();
my %cvs_names = ();

my ($fcnt, $ccnt, $msg);
my $mxlen = 0;
my $mxsiz = 0;
my $szlen = 0;
my $latest = 0;
my $largest = 0;
my $most_recent = 0;
my $recent_file = '';

# options
my $exclude_built = 1;

my @excluded_ext = qw( .old .bak .obj .err .pdb .lst .pch .ilk 
    .ncb .plg .OPT .idb .aps .sbr .suo .bsc .manifest .user
    .res .zip .exe .lib .dll .exp
);
my @excluded_vc = qw( .dsw .dsp .sln .vcproj );

my @exclude_others = qw( temp*.* mt.dep Buildlog.htm );

# debug
my $dbg1 = 0;	# show processing folders ...
my $dbg2 = 0;	# show CVS/SVN file entries
my $dbg3 = 0;	# show CVS/SVN file types

parse_args(@ARGV);

prt( "Processing $in_folder ...\n" ) if ($dbg1);
process_dir( $in_folder, 0 );
$fcnt = scalar @fnd_files;
$ccnt = scalar @fnd_cvs;
prt( "Found $fcnt files, $ccnt in CVS/SVN folders ...\n" );

my $alfildate = show_latest_file();

for (my $i = 0; $i < $ccnt; $i++) {
	my $ff = $fnd_cvs[$i][0];
	my $sz = $fnd_cvs[$i][2];
	my $sf = substr($ff, (length($in_folder) + 1));
	my $len = length($sf);
	if ($len > $mxlen) {
		$mxlen = $len;
	}
	if ($sz > $mxsiz) {
		$mxsiz = $sz;
	}
}
$msg = get_nn($mxsiz);
$szlen = length($msg) + 1;

for (my $i = 0; $i < $ccnt; $i++) {
	my $ff = $fnd_cvs[$i][0];
	my $tm = $fnd_cvs[$i][1];
	my $sz = $fnd_cvs[$i][2];
	my $sf = substr($ff, (length($in_folder) + 1));
	###my ($nm, $dir, $ext) = fileparse( $sf, qr/\.[^.]*/ );
	my ($nm, $dir) = fileparse( $sf );
	if (defined $cvs_names{$nm}) {
		$cvs_names{$nm}++;
	} else {
		$cvs_names{$nm} = 1;
	}
	if ($sz > $largest) {
		$largest = $sz;
	}
	my $nsz = get_nn($sz);
	while(length($nsz) < $szlen) {
		$nsz = ' ' . $nsz;
	}
	$msg = $sf;
	while(length($msg) < $mxlen) {
		$msg .= ' ';
	}
	if ($tm > $latest) {
		$latest = $tm;
	}
	prt( "$msg ". localtime($tm). " $nsz\n" ) if ($dbg2);
}

foreach my $key (keys %cvs_names) {
	my $val = $cvs_names{$key};
	prt( "$key $val\n" ) if ($dbg3);
}

my $nsz = get_nn($largest);
while(length($nsz) < $szlen) {
	$nsz = ' ' . $nsz;
}
$msg = "Latest (largest)";
##while(length($msg) < $mxlen) {
##	$msg .= ' ';
##}
##prt( "$msg ". localtime($latest). " $nsz\n" );
if ($latest) {
    prt( "$msg ". get_YYYYMMDD_hhmmss_UTC($latest) . " $nsz\n" );
    prt( "CVS/SVN: ".get_YYYYMMDD_UTC($latest) );
}  else {
    prt( "$msg NO LATEST CVS/SVN TIME $nsz\n" );
    prt( "NO CVS/SVN FILES" );
}
prt( ", FILE: $alfildate" );
if ($most_recent > 0) {
    prt( ", MOST RECENT: ".get_YYYYMMDD_UTC($most_recent). " $recent_file" );
}
prt("\n");

exit(0);
##########################
##### subs
sub in_excluded_vc {
    my ($lcext) = shift;
    foreach my $ext (@excluded_vc) {
        if ($lcext eq $ext) {
            return 1;
        }
    }
    return 0;
}

sub in_excluded_ext {
    my ($lcext) = shift;
    foreach my $ext (@excluded_ext) {
        if ($lcext eq $ext) {
            return 1;
        }
    }
    return 0;
}

sub in_excluded {
    my ($lcext) = shift;
    return 1 if in_excluded_ext($lcext);
    return 1 if in_excluded_vc($lcext);
    return 0;
}

sub show_latest_file {
    my $max = scalar @fnd_files;
    my ($i, $ff, $tm, $sz, $fn);
    my ($nam, $dir, $ext, $name);
    my $lasttime = 0;
    my $lastfile = '';
    my $lastsize = 0;
    my $rmsg = '';
    for ($i = 0; $i < $max; $i++) {
        $ff = $fnd_files[$i][0];
        $tm = $fnd_files[$i][1];
        $sz = $fnd_files[$i][2];
        $fn = substr($ff, (length($in_folder) + 1));
        if ($tm > $most_recent) {
            $most_recent = $tm;
            $recent_file = $fn;
        }
        if ($exclude_built) {
            ($nam,$dir,$ext) = fileparse($ff, qr/\.[^.]*/ );
            next if in_excluded(lc($ext));
            $name = $nam . $ext;
            next if ($name =~ /^temp/i);
            next if (lc($name) eq 'mt.dep');
            next if (lc($name) eq 'buildlog.htm' );
        }
        if ($tm > $lasttime) {
            $lasttime = $tm;
            $lastfile = $fn;
            $lastsize = $sz;
        }
    }
    if ($lasttime) {
        $rmsg = get_YYYYMMDD_UTC($lasttime). " $lastfile";
        prt( "Latest file [$lastfile], [". get_YYYYMMDD_hhmmss_UTC($lasttime) ."], size ".get_nn($lastsize)."\n" );
    } else {
        $rmsg = "None found!";
        prt( "No latest file found!???\n" );
    }
    return $rmsg;
}

sub prt {
	my ($m) = shift;
	print $m;
}

sub mydie {
	my ($m) = shift;
	die $m;
}

sub process_dir {
	my ($inf, $lev) = @_;
	prt( "$lev: Processing $inf folder ...\n" ) if ($dbg1);
	if ( opendir( DIR, $inf ) ) {
		my @files = readdir(DIR);
		closedir DIR;
		foreach my $fl (@files) {
			next if (($fl eq '.') || ($fl eq '..'));
			my $ff = $inf . "/" . $fl;
			if (-d $ff) {
				process_dir( $ff, ($lev + 1) ); # process this sub-directory
			} else {
				my $sb = stat($ff);
				if (($ff =~ /\/CVS\//i)||($ff =~ /\/\.svn\//i)) {
					push(@fnd_cvs, [$ff, $sb->mtime, $sb->size, $lev] );
				} else {
					push(@fnd_files, [$ff, $sb->mtime, $sb->size, $lev] );
				}
			}
		}
	} else {
		prt( "WARNING: Can NOT open $inf ... $! ...\n" );
	}
}

sub give_help {
	prt( "$0 [OPTIONS] in_folder\n" );
	prt( "OPTIONS:\n" );
	prt( " -? or -h - This brief help.\n" );
	prt( " -d1      - show processing folders.\n" );
	prt( " -d2      - show CVS/SVN file entries.\n" );
	prt( " -d3      - show CVS/SVN file types.\n" );
	mydie("In folder must exist ...\n");
}

sub parse_args { # @ARGV
	my (@av) = @_;
    my ($dn, @tv);
	while (@av) {
		my $arg = $av[0];
		if (substr($arg,0,1) eq '-') {
			if (($arg eq '-?')||($arg eq '-h')) {
				give_help();
			} elsif ($arg eq '-d1') {
				$dbg1 = 1;
				prt( "Show folder processing ...\n" );
			} elsif ($arg eq '-d2') {
				$dbg2 = 1;
				prt( "Show CVS/SVN file entries ...\n" );
			} elsif ($arg eq '-d3') {
				$dbg3 = 1;
				prt( "Show CVS/SVN file types ...\n" );
            } elsif ($arg =~ /^-d(\d+)/) {
                $dn = $1;
                @tv = ();
                push(@tv,'-d3') if ($dn >= 3);
                push(@tv,'-d2') if ($dn >= 2);
                push(@tv,'-d1') if ($dn >= 1);
                parse_args(@tv) if (@tv);
			} else {
				mydie( "ERROR: Unknown option [$arg] ... aborting ...\n" );
			}
		} else {
			# bare item - assume INPUT folder
			$in_folder = dos_2_unix($arg);
			prt( "Set in folder to $in_folder ...\n" );
		}

		shift @av;
	}
}

sub dos_2_unix($) {
	my ($du) = shift;
	$du =~ s/\\/\//g;
	return $du;
}

##################################################
# My particular 'nice number'
sub get_nn { # perl nice number nicenum add commas
	my ($n) = shift;
	if (length($n) > 3) {
		my $mod = length($n) % 3;
		my $ret = (($mod > 0) ? substr( $n, 0, $mod ) : '');
		my $mx = int( length($n) / 3 );
		for (my $i = 0; $i < $mx; $i++ ) {
			if (($mod == 0) && ($i == 0)) {
				$ret .= substr( $n, ($mod+(3*$i)), ($mod+(3*$i)+3) );
			} else {
				$ret .= ',' . substr( $n, ($mod+(3*$i)), ($mod+(3*$i)+3) );
			}
		}
		return $ret;
	}
	return $n;
}

sub get_YYYYMMDD_hhmmss_UTC {
    my ($t) = shift;
    # sec, min, hour, mday, mon, year, wday, yday, and isdst.
    my $tm = gmtime($t);
    my $m = sprintf( "%04d/%02d/%02d %02d:%02d:%02d",
        $tm->year() + 1900, $tm->mon() + 1, $tm->mday(), $tm->hour(), $tm->min(), $tm->sec());
    return $m;
}

sub get_YYYYMMDD_UTC {
    my ($t) = shift;
    # sec, min, hour, mday, mon, year, wday, yday, and isdst.
    my $tm = gmtime($t);
    my $m = sprintf( "%04d/%02d/%02d",
        $tm->year() + 1900, $tm->mon() + 1, $tm->mday());
    return $m;
}


# eof - cvsversion.pl
